#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright (C) 2021 Martin Hasin - License: GNU General Public License v2
# Contact: martin.hasin@gmail.com

from cmk.gui.i18n import _
from cmk.gui.valuespec import (
        Dictionary,
        Integer,
        Tuple,
        TextAscii,
        )
from cmk.gui.plugins.wato import (
        CheckParameterRulespecWithItem,
        CheckParameterRulespecWithoutItem,
        rulespec_registry,
        RulespecGroupCheckParametersEnvironment,
        )

def _parameter_valuespec_shelly_power():
    return Dictionary (
            title = _("SHELLY EM power via API"),
            optional_keys = ["power", "reactive", "voltage"],
            elements = [
                ("L-power", Tuple (
                    title = _("L1 + L2 + L3 Power Wat"),
                    elements = [
                        Integer(title = _("Warning below"), default_value = 15),
                        Integer(title = _("Critical higher"), default_value = 800),
                    ],
                    )),



                ("L1-power", Tuple (
                    title = _("L1 Power Wat"),
                    elements = [
                        Integer(title = _("Warning below"), default_value = 15),
                        Integer(title = _("Critical higher"), default_value = 800),
                    ],
                    )),
                ("L1-reactive", Tuple (
                    title = _("L1 Power VA reactive"),
                    elements = [
                        Integer(title = _("Warning below"), default_value = 20),
                        Integer(title = _("Critical higher"), default_value = 900),
                        ],
                    )),
                ("L1-voltage", Tuple (
                    title = _("L1 Voltage range"),
                    elements = [
                        Integer(title = _("Warning below"), default_value = 190),
                        Integer(title = _("Critical higher"), default_value = 250),
                        ],
                    )),

                ("L1-current", Tuple (
                    title = _("L1 Current range"),
                    elements = [
                        Integer(title = _("Warning below"), default_value = 190),
                        Integer(title = _("Critical higher"), default_value = 250),
                        ],
                    )),



                ("L2-power", Tuple (
                    title = _("L2 Power Wat"),
                    elements = [
                        Integer(title = _("Warning below"), default_value = 15),
                        Integer(title = _("Critical higher"), default_value = 800),
                    ],
                    )),
                ("L2-reactive", Tuple (
                    title = _("L2 Power VA reactive"),
                    elements = [
                        Integer(title = _("Warning below"), default_value = 20),
                        Integer(title = _("Critical higher"), default_value = 900),
                        ],
                    )),
                ("L2-voltage", Tuple (
                    title = _("L2 Voltage range"),
                    elements = [
                        Integer(title = _("Warning below"), default_value = 190),
                        Integer(title = _("Critical higher"), default_value = 250),
                        ],
                    )),

                ("L2-current", Tuple (
                    title = _("L2 Current range"),
                    elements = [
                        Integer(title = _("Warning below"), default_value = 190),
                        Integer(title = _("Critical higher"), default_value = 250),
                        ],
                    )),


                ("L3-power", Tuple (
                    title = _("L3 Power Wat"),
                    elements = [
                        Integer(title = _("Warning below"), default_value = 15),
                        Integer(title = _("Critical higher"), default_value = 800),
                    ],
                    )),
                ("L3-reactive", Tuple (
                    title = _("L3 Power VA reactive"),
                    elements = [
                        Integer(title = _("Warning below"), default_value = 20),
                        Integer(title = _("Critical higher"), default_value = 900),
                        ],
                    )),
                ("L3-voltage", Tuple (
                    title = _("L3 Voltage range"),
                    elements = [
                        Integer(title = _("Warning below"), default_value = 190),
                        Integer(title = _("Critical higher"), default_value = 250),
                        ],
                    )),


                ("L3-current", Tuple (
                    title = _("L3 Current range"),
                    elements = [
                        Integer(title = _("Warning below"), default_value = 190),
                        Integer(title = _("Critical higher"), default_value = 250),
                        ],
                    )),

                ("Ltotal_current", Tuple (
                    title = _("L1 + L2 + L3 Current range"),
                    elements = [
                        Integer(title = _("Warning below"), default_value = 190),
                        Integer(title = _("Critical higher"), default_value = 250),
                        ],
                    )),

                ("Ltotal_returned", Tuple (
                    title = _("L1 + L2 + L3 total returned"),
                    elements = [
                        Integer(title = _("Warning below"), default_value = 190),
                        Integer(title = _("Critical higher"), default_value = 250),
                        ],
                    )),




                ],

            )

rulespec_registry.register (
        CheckParameterRulespecWithoutItem(
            check_group_name = "shelly_power_data",
            group = RulespecGroupCheckParametersEnvironment,
            match_type = "dict",
            parameter_valuespec = _parameter_valuespec_shelly_power,
            title = lambda: _("SHELLY EM power via API"),
            )
        )

