#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright (C) 2021 Martin Hasin - License: GNU General Public License v2
# Contact: martin.hasin@gmail.com

# Agent output:
#<<<shelly_power>>>
#NAME : examplename
#L1_power : 246.43
#L1_reactive : 192.75
#L1_voltage : 238.84
#L1_total_returned : 1126125.0
#L1_total : 6901976.2
#L2_power : 246.43
#L2_reactive : 192.75
#L2_voltage : 238.84
#L2_total_returned : 1126125.0
#L2_total : 6901976.2
#L3_power : 246.43
#L3_reactive : 192.75
#L3_voltage : 238.84
#L3_total_returned : 1126125.0
#L3_total : 6901976.2

from .agent_based_api.v1 import *
import pprint

def parse_shellypower(string_table):
    shellyname = "EM"
#    pprint(string_table)
    name = " "
    parsed = []
    for line in string_table:
        if line[0] == "NAME":
            name = line[2]
        if line[0] in ["STATUS", "L1_power", "L1_reactive", "L1_voltage","L1_pf","L1_current", "L2_power", "L2_reactive", "L2_voltage","L2_pf","L2_current","L3_power", "L3_reactive", "L3_voltage","L3_pf","L3_current","L1_total", "L2_total", "L3_total","L1_total_returned", "L2_total_returned", "L3_total_returned", "L_total", "L_total_current", "L_total_returned", "L_total_energy"]:
            element = []
            description = "UNKNOWN"
            if (line[0] == "STATUS"):
               description = "Status"
            elif (line[0] == "L1_power"):
                description = "L1 Power"
            elif (line[0] == "L1_reactive"):
                description = "L1 Reactive"
            elif (line[0] == "L1_voltage"):
                description = "L1 Voltage"
            elif (line[0] == "L1_pf"):
                description = "L1 PF"
            elif (line[0] == "L1_current"):
                description = "L1 Current"

            elif (line[0] == "L2_power"):
                description = "L2 Power"
            elif (line[0] == "L2_reactive"):
                description = "L2 Reactive"
            elif (line[0] == "L2_voltage"):
                description = "L2 Voltage"
            elif (line[0] == "L2_pf"):
                description = "L2 PF"
            elif (line[0] == "L2_current"):
                description = "L2 Current"

            elif (line[0] == "L3_power"):
                description = "L3 Power"
            elif (line[0] == "L3_reactive"):
                description = "L3 Reactive"
            elif (line[0] == "L3_voltage"):
                description = "L3 Voltage"
            elif (line[0] == "L3_pf"):
                description = "L3 PF"
            elif (line[0] == "L3_current"):
                description = "L3 Current"


            elif (line[0] == "L1_total"):
                description = "L1 total energy"

            elif (line[0] == "L2_total"):
                description = "L2 total energy"
            elif (line[0] == "L3_total"):
                description = "L3 total energy"
            elif (line[0] == "L1_total_returned"):
                description = "L1 total returned energy"

            elif (line[0] == "L2_total_returned"):
                description = "L2 total returned energy"
            elif (line[0] == "L3_total_returned"):
                description = "L3 total returned energy"
            elif (line[0] == "L_total"):
                description = "L1 + L2 + L3 total energy"

            elif (line[0] == "L_total_current"):
                description = "L1 + L2 + L3 total Current"

            elif (line[0] == "L_total_returned"):
                description = "L1 + L2 + L3 total returned energy"

            elif (line[0] == "L_total_energy"):
                description = "L1 + L2 + L3 total consumed energy"



            element.append(shellyname + " " + name + " " + description)
            element.append(line[0])
            element.append(line[2])
            parsed.append(element)
    return parsed

def discover_shellypower(section):
    for line in section:
        yield Service(item=line[0])

def check_shellypower(item, params, section):
#    pprint(item)
#    pprint(section)
    L1power_warn = params['L1-power'][0]
    L1power_crit = params['L1-power'][1]
    L1reactive_warn = params['L1-reactive'][0]
    L1reactive_crit = params['L1-reactive'][1]
    L1voltage_warn = params['L1-voltage'][0]
    L1voltage_crit = params['L1-voltage'][1]

    L1current_warn = params['L1-current'][0]
    L1current_crit = params['L1-current'][1]

    L2power_warn = params['L2-power'][0]
    L2power_crit = params['L2-power'][1]
    L2reactive_warn = params['L2-reactive'][0]
    L2reactive_crit = params['L2-reactive'][1]
    L2voltage_warn = params['L2-voltage'][0]
    L2voltage_crit = params['L2-voltage'][1]
    L2current_warn = params['L2-current'][0]
    L2current_crit = params['L2-current'][1]


    L3power_warn = params['L3-power'][0]
    L3power_crit = params['L3-power'][1]
    L3reactive_warn = params['L3-reactive'][0]
    L3reactive_crit = params['L3-reactive'][1]
    L3voltage_warn = params['L3-voltage'][0]
    L3voltage_crit = params['L3-voltage'][1]
    L3current_warn = params['L3-current'][0]
    L3current_crit = params['L3-current'][1]
    Lpower_warn = params['L-power'][0]
    Lpower_crit = params['L-power'][1]

    Ltotal_current_warn = params['Ltotal_current'][0]
    Ltotal_current_crit = params['Ltotal_current'][1]



    for sector, key, val in section:
        if (sector == item):
            s = State.UNKNOWN;
            report = "not found"
            if (key == "STATUS"):
                # Valid Status: CAL TRIM BOOST ONLINE ONBATT OVERLOAD LOWBATT REPLACEBATT NOBATT SLAVE SLAVEDOWN COMMLOST or SHUTTING DOWN
                report = val
                if (val == "ONLINE"):
                    s = State.OK
                elif (val in ["CAL", "TRIM", "BOOST"]):
                    s = State.WARNING
                elif (val in ["ONBATT", "OVERLOAD", "ONBATT", "REPLACEBATT", "NOBATT"]):
                    s = State.CRITICAL
            elif (key == "L1_power"):
                val = float(val)
                report = str(val) + " W"
                if (val >= L1power_crit):
                    s = State.CRIT
                elif (val <= L1power_warn):
                    s = State.WARN
                else:
                    s = State.OK
                yield Metric( "L1-power", val, boundaries=(0, 1000), levels=(L1power_warn, L1power_crit))
            elif (key == "L1_reactive"):
                val = float(val)
                report = str(val) + " VAR"
                if (val >= L1reactive_crit):
                    s = State.CRIT
                elif (val <= L1reactive_warn):
                    s = State.WARN
                else:
                    s = State.OK
                yield Metric( "L1-reactive", val, boundaries=(0, 100), levels=(L1reactive_warn, L1reactive_crit))
            elif (key == "L1_voltage"):
                val = float(val)
                report = str(val) + " V"
                if (val >= L1voltage_crit):
                    s = State.CRIT
                elif (val <= L1voltage_warn):
                    s = State.WARN
                else:
                    s = State.OK
                yield Metric( "L1-voltage", val, boundaries=(120, 240),levels=(L1voltage_warn, L1voltage_crit))

            elif (key == "L1_pf"):
                val = float(val)
                report = str(val) 
                s = State.OK
                yield Metric( "L1-pf", val, boundaries=(0, 1))

            elif (key == "L1_current"):
                val = float(val)
                report = str(val) + " A"
                if (val >= L1current_crit):
                    s = State.CRIT
                elif (val <= L1current_warn):
                    s = State.WARN
                else:
                    s = State.OK
                yield Metric( "L1-current", val, boundaries=(0, 50),levels=(L1current_warn, L1current_crit))



            elif (key == "L2_power"):
                val = float(val)
                report = str(val) + " W"
                if (val >= L2power_crit):
                    s = State.CRIT
                elif (val <= L2power_warn):
                    s = State.WARN
                else:
                    s = State.OK
                yield Metric( "L2-power", val, boundaries=(0, 100), levels=(L2power_warn, L2power_crit))
            elif (key == "L2_reactive"):
                val = float(val)
                report = str(val) + " VAR"
                if (val >= L2reactive_crit):
                    s = State.CRIT
                elif (val <= L2reactive_warn):
                    s = State.WARN
                else:
                    s = State.OK
                yield Metric( "L2-reactive", val, boundaries=(0, 100), levels=(L2reactive_warn, L2reactive_crit))
            elif (key == "L2_voltage"):
                val = float(val)
                report = str(val) + " V"
                if (val >= L2voltage_crit):
                    s = State.CRIT
                elif (val <= L2voltage_warn):
                    s = State.WARN
                else:
                    s = State.OK
                yield Metric( "L2-voltage", val, boundaries=(120, 240),levels=(L2voltage_warn, L2voltage_crit))

            elif (key == "L2_pf"):
                val = float(val)
                report = str(val)
                s = State.OK
                yield Metric( "L2-pf", val, boundaries=(0, 1))

            elif (key == "L2_current"):
                val = float(val)
                report = str(val) + " A"
                if (val >= L2current_crit):
                    s = State.CRIT
                elif (val <= L2current_warn):
                    s = State.WARN
                else:
                    s = State.OK
                yield Metric( "L2-current", val, boundaries=(0, 50),levels=(L2current_warn, L2current_crit))





            elif (key == "L3_power"):
                val = float(val)
                report = str(val) + " W"
                if (val >= L3power_crit):
                    s = State.CRIT
                elif (val <= L3power_warn):
                    s = State.WARN
                else:
                    s = State.OK
                yield Metric( "L3-power", val, boundaries=(0, 100), levels=(L3power_warn, L3power_crit))
            elif (key == "L3_reactive"):
                val = float(val)
                report = str(val) + " VAR"
                if (val >= L3reactive_crit):
                    s = State.CRIT
                elif (val <= L3reactive_warn):
                    s = State.WARN
                else:
                    s = State.OK
                yield Metric( "L3-reactive", val, boundaries=(0, 100), levels=(L3reactive_warn, L3reactive_crit))
            elif (key == "L3_voltage"):
                val = float(val)
                report = str(val) + " V"
                if (val >= L3voltage_crit):
                    s = State.CRIT
                elif (val <= L3voltage_warn):
                    s = State.WARN
                else:
                    s = State.OK
                yield Metric( "L3-voltage", val, boundaries=(120, 240),levels=(L3voltage_warn, L3voltage_crit))

            elif (key == "L3_pf"):
                val = float(val)
                report = str(val) 
                s = State.OK
                yield Metric( "L3-pf", val, boundaries=(0, 1))




            elif (key == "L1_total"):
                val = float(val)
                report = str(val) + " W/h"
                s = State.OK
                yield Metric( "L1_total", val, boundaries=(0, 1))
            elif (key == "L2_total"):
                val = float(val)
                report = str(val) + " W/h"
                s = State.OK
                yield Metric( "L2_total", val, boundaries=(0, 1))
            elif (key == "L3_total"):
                val = float(val)
                report = str(val) + " W/h"
                s = State.OK
                yield Metric( "L3_total", val, boundaries=(0, 1))

            elif (key == "L3_current"):
                val = float(val)
                report = str(val) + " A"
                if (val >= L3current_crit):
                    s = State.CRIT
                elif (val <= L3current_warn):
                    s = State.WARN
                else:
                    s = State.OK
                yield Metric( "L3-current", val, boundaries=(0, 50),levels=(L3current_warn, L3current_crit))




            elif (key == "L1_total_returned"):
                val = float(val)
                report = str(val) + " W/h"
                s = State.OK
                yield Metric( "L1_total_returned", val, boundaries=(120, 240))
            elif (key == "L2_total_returned"):
                val = float(val)
                report = str(val) + " W/h"
                s = State.OK
                yield Metric( "L2_total_returned", val, boundaries=(120, 240))
            elif (key == "L3_total_returned"):
                val = float(val)
                report = str(val) + " W/h"
                s = State.OK
                yield Metric( "L3_total_returned", val, boundaries=(120, 240))

            elif (key == "L_total"):
                val = float(val)
                report = str(val) + " W"
                if (val >= L1power_crit):
                    s = State.CRIT
                elif (val <= L1power_warn):
                    s = State.WARN
                else:
                    s = State.OK
                yield Metric( "L-power", val, boundaries=(0, 1000), levels=(Lpower_warn, Lpower_crit))



            elif (key == "L_total_current"):
                val = float(val)
                report = str(val) + " A"
                if (val >= Ltotal_current_crit):
                    s = State.CRIT
                elif (val <= Ltotal_current_warn):
                    s = State.WARN
                else:
                    s = State.OK
                yield Metric( "L_total_current", val, boundaries=(0, 1000), levels=(Ltotal_current_warn, Ltotal_current_crit))


            elif (key == "L_total_returned"):
                val = float(val) 
                report = str(val) + " W/h"
                s = State.OK
                yield Metric( "L_total_returned", val, boundaries=(0, 1000))



            elif (key == "L_total_energy"):
                val = float(val)
                report = str(val) + " W/h"
                s = State.OK
                yield Metric( "L_total_energy", val, boundaries=(0, 1000))




            yield Result(state=s, summary=report)
            return

register.agent_section(
        name="shelly_power_data",
        parse_function=parse_shellypower,
)

register.check_plugin(
        name = "shelly_power_data",
        service_name = "SHELLY %s",
        sections=["shelly_power_data"],
        discovery_function = discover_shellypower,
        check_function = check_shellypower,
        check_default_parameters = {
            'L1-power': (15, 800),
            'L1-reactive': (20, 900),
            'L1-voltage': (190, 250),
            'L1-current': (190, 250),
            'L2-power': (15, 800),
            'L2-reactive': (20, 900),
            'L2-voltage': (190, 250),
            'L2-current': (190, 250),
            'L3-power': (15, 800),
            'L3-reactive': (20, 900),
            'L3-voltage': (190, 250),
            'L3-current': (190, 250),
            'L-power': (15, 800),
            'Ltotal_current': (1, 20)},

        check_ruleset_name = "shelly_power_data",
)
